@@ -1,5 +1,5 @@
-from ..utils.initializers import get_initializer
-from xshinnosuke.layers.activators import get_activator
+from ..nn.initializers import get_initializer
+from .activators import get_activator
 from ..nn.core import Layer, Variable
 from ..nn import global_graph as GlobalGraph
 from ..nn.functional import conv2d, max_pool2d, avg_pool2d
