@@ -1,7 +1,7 @@
-from ..nn.core import Variable
-from ..nn.grad_fn import *
-from ..utils.toolkit import im2col, initialize_ops_grad
-from ..utils.initializers import Zeros, Ones
+from .core import Variable
+from .grad_fn import *
+from .toolkit import im2col, initialize_ops_grad
+from xshinnosuke.nn.initializers import Zeros, Ones
 from typing import Tuple
 
 
@@ -45,13 +45,12 @@ def softmax(inputs: Variable):
 
 
 def dense(inputs: Variable, weight: Variable, bias: Variable = None):
-    outputs = inputs.matmul(weight)
+    outputs = inputs.data.dot(weight.data)
     if bias is not None:
-        outputs += bias
-    else:
-        # 这里加入None是为了数组大小能对应
-        outputs.in_bounds.append(None)
+        outputs += bias.data
+    outputs = Variable(data=outputs, in_bounds=[inputs, weight, bias])
     outputs.grad_fn = DenseBackward
+    inputs.out_bounds.append(outputs)
     initialize_ops_grad(inputs, weight, bias)
     return outputs
 
@@ -239,9 +238,6 @@ def batchnorm_2d(inputs: Variable, gamma: Variable, beta: Variable, axis: int, e
     return outputs
 
 
-
-
-
 def concatenate(*variables: Variable, axis: int, output: Variable = None, name: str = None):
     data = variables[0].data
     for i in range(1, len(variables)):
